

## Introduction and general remarks

### What is this README about?

> Introduction to the code structure and dependencies



### Conda virtual environment:

We used a conda environment on Linux Debian Version 9.
Use the provided torch_env.yml file to create this environment as follows:

conda env create --name torch_env --file=torch_env.yml


## ML Reproducibility

### Datasets

##### A link to a downloadable version of the dataset and simulation environment.

In order to rerun our evaluation, you need to add the following UCI datasets 
to the "data" folder that is located in the same directory as the Jupyter notebooks.

concrete
http://archive.ics.uci.edu/ml/datasets/Concrete+Compressive+Strength
Put Concrete_Data.xls into the data folder.

energy
https://archive.ics.uci.edu/ml/datasets/Energy+efficiency
Put ENB2012_data.xlsx into the data folder.

abalone
https://archive.ics.uci.edu/ml/datasets/abalone
Put abalone.data and abalone.names into the data folder.

naval
https://archive.ics.uci.edu/ml/datasets/Condition+Based+Maintenance+of+Naval+Propulsion+Plants
Extract the UCI CBM Dataset.zip file into the data folder, such that the data folder has a subdirectory "UCI CBM Dataset"

power
https://archive.ics.uci.edu/ml/datasets/Combined+Cycle+Power+Plant
Extract the CCPP.zip file into the data folder, such that the data folder has a subdirectory CCPP.

protein
https://archive.ics.uci.edu/ml/datasets/Physicochemical+Properties+of+Protein+Tertiary+Structure
Put CASP.csv into the data folder.

wine quality
https://archive.ics.uci.edu/ml/datasets/wine+quality
Put winequality-red.csv into the data folder.

yacht
http://archive.ics.uci.edu/ml/datasets/yacht+hydrodynamics
Put yacht_hydrodynamics.data into the data folder.

year
https://archive.ics.uci.edu/ml/datasets/YearPredictionMSD
Extract YearPredictionMSD.txt.zip into the data folder such that the data folder has the File "YearPredictionMSD.txt"

superconduct
https://archive.ics.uci.edu/ml/datasets/Superconductivty+Data
Extract superconduct.zip into the data folder, such that the data folder has a subdirectory "superconduct"

boston, diabetes, california
These are obtained using functions from sklearn, namely
load_boston, load_diabetes, fetch_california_housing from the sklear.datasets package.

The toy datasets are generated using functions in the model_training_and_evaluation.ipybn notebook.

### Code

For all shared code related to this work, check if you include:

##### Specification of dependencies

Specified by the supplied torch_env.yml file.
The code should run as it is given in an conda environment as described above.

##### Training and Evaluation code

The code for the experiments on the UCI and toy datasets is provided in the jupyter notebook: model_training_and_evaluation.ipynb
Make sure to have the conda environment activated. Then execute all cells except the last one.

In the last cell:
Insert a path at line 24 (exp_dir= ...) The path points to a directory EXP_DIR in which experiment logfiles are stored
If the directory does not exist it will be created.

To run the training/evaluation for the hyperparameter study (beta parameter) replace the line
methods = available_methods with
methods = { 'mc_mod_sml1', 'mc_mod_sml25', 'mc_mod_sml', 'mc_mod_sml75', 'mc_mod_sml90'}
This will start the training with SML Loss only and different values for beta.

Run the last cell to start the experiment. After the training finishes, EXP_DIR contains several subdirectories for each dataset.
Those subdirectories contain zipped dictionaries that contain the performance/uncertainty metrics on train/test data.

The visualize the evaluation, another notebook is provided: model_evaluation_and_plots.ipynb

First run cells 1-4.

To create plots of Figure 12, first run cells 5, 6. 
Set the path in the exp_dirs list. This should point to the EXP_DIR used when running the training/evaluation script.
Finally, run cell 7. The plots will be stored to the ./plots directory.

To create plots of Figure 2, first run cells 8, 9.
In cell 10 change the exp_dir path accordingly. It should point to EXP_DIR that contains a subdirectory with the toy_modulated dataset.
Run cell 10 to create the plots for toy_modulated. It will be stored to ./plots/toy_modulated.pdf
In cell 11 change the exp_dir path accordingly. It should point to EXP_DIR that contains a subdirectory with the toy_hf dataset.
Run cell 11 to create the plot for toy_hf. It will be stored to ./plots/toy_hf.pdf

To create plots of Figures 3, 8, 9, 10, first run cells 12, 13, 14.
In cell 15 change the exp_dirs list accordingly. It should contain paths to EXP_DIRs.
Run cells 15 afterwards.
Run cell 18 to create a plot showing RMSE performance on all datasets. Run cell 19 to include SML_DE in the plot.
Run cell 21 to create a plot showing NLL performance on all datasets. Run cell 22 to include SML_DE in the plot.
Run cell 24 to create a plot showing ECE uncertainty on all datasets. Run cell 25 to include SML_DE in the plot.
Run cell 27 to create a plot showing Wasserstein distance on all datasets. Run cell 28 to include SML_DE in the plot.
All plots will be stored to the ./plots directory.

To create the plots of Figures 13, 14, change the exp_dirs list of cell 29 accordingly. It should contain paths to EXP_DIR of a hyperparameter trainings/evaluation run as described above.
Then run cells 29-33 to create the plots for the hyperparameter study with regards to RMSE, NLL, ECE and Wasserstein distance.
The plots will be stored to the ./plots directory.

To create the plot of Figure 15 first run cell 34. Again change the exp_dirs list of cell 35 accordingly. Pointing to EXP_DIRs of a standard training/evaluation run.
Then run cell 35, 38, 39, 40 to create the plots. They will be stored to the ./plots directory.


##### Pre-trained model(s)

Not applicable, i.e., we don't use pre-trained models for the UCI training.

##### README file includes table of results accompanied by precise command to run to produce those results

This readme file includes the commands to run the experiments.
For "table" of results see Table 3, 5, 6.

Rerun the experiments using the supplied jupyter notebooks as described in "Training and Evaluation code".
Note that, since train-test-splits and initializations are randomly drawn, there is no guarantuee to get the exact same results as in the paper.

### Experimental Results

For all reported experimental results, check if you include:

##### The range of hyper-parameters considered, method to select the best hyper-parameter configuration, and specification of all hyper-parameters used to generate results.

See the jupyter notebooks for the hyperparameters used to run the experiments and obtain the results.

##### The average runtime for each result, or estimated energy cost.

Conducting the described experiments with cross validation takes 6h (on CPU) for toy data, 80h (on CPU) for UCI regression and 8h (on GPU) for object regression.

##### A description of the computing infrastructure used.

All experiments are conducted on a
> Intel(R) Xeon(R) Gold 6126 CPU @ 2.60GHz
> For object-detection we used additionally a Tesla V100 GPU.